"""Test than modules in eventlet.green package are indeed green.
To do that spawn a green server and then access it using a green socket.
If either operation blocked the whole script would block and timeout.
"""
import unittest

from eventlet.green import BaseHTTPServer
from eventlet import spawn, kill
from eventlet.support import six

if six.PY2:
    from eventlet.green.urllib2 import HTTPError, urlopen
else:
    from eventlet.green.urllib.request import urlopen
    from eventlet.green.urllib.error import HTTPError


class QuietHandler(BaseHTTPServer.BaseHTTPRequestHandler):
    protocol_version = "HTTP/1.0"

    def log_message(self, *args, **kw):
        pass


def start_http_server():
    server_address = ('localhost', 0)
    httpd = BaseHTTPServer.HTTPServer(server_address, QuietHandler)
    sa = httpd.socket.getsockname()
    # print("Serving HTTP on", sa[0], "port", sa[1], "...")
    httpd.request_count = 0

    def serve():
        # increment the request_count before handling the request because
        # the send() for the response blocks (or at least appeared to be)
        httpd.request_count += 1
        httpd.handle_request()
    return spawn(serve), httpd, sa[1]


class TestGreenness(unittest.TestCase):

    def setUp(self):
        self.gthread, self.server, self.port = start_http_server()
        # print('Spawned the server')

    def tearDown(self):
        self.server.server_close()
        kill(self.gthread)

    def test_urllib(self):
        self.assertEqual(self.server.request_count, 0)
        try:
            urlopen('http://127.0.0.1:%s' % self.port)
            assert False, 'should not get there'
        except HTTPError as ex:
            assert ex.code == 501, repr(ex)
        self.assertEqual(self.server.request_count, 1)

if __name__ == '__main__':
    unittest.main()



from eventlet.green import threading, time
from eventlet.green.http import client
from eventlet.green.urllib import parse as urllib_parse, request as urllib_request
from eventlet import patcher

patcher.inject('http.cookiejar', globals(),
               ('http.client', client), ('threading', threading),
               ('urllib.parse', urllib_parse), ('urllib.request', urllib_request),
               ('time', time))

del urllib_request
del urllib_parse
del patcher



from eventlet import patcher
from eventlet.green import os, socket
from eventlet.green.urllib import parse as urllib_parse

patcher.inject('http.client', globals(),
               ('os', os), ('socket', socket), ('urllib.parse', urllib_parse))

del patcher
del urllib_parse



__test__ = False

if __name__ == '__main__':
    import sys
    import eventlet
    try:
        from dns import reversename
    except ImportError:
        print('skip:require dns (package dnspython)')
        sys.exit(1)
    eventlet.monkey_patch(all=True)
    reversename.from_address('127.0.0.1')
    print('pass')



import eventlet
from eventlet.green import socket


def test_create_connection_error():
    try:
        socket.create_connection(('192.0.2.1', 80), timeout=0.1)
    except (IOError, OSError):
        pass


def test_recv_type():
    # https://github.com/eventlet/eventlet/issues/245
    # socket recv returning multiple data types
    # For this test to work, client and server have to be in separate
    # processes or OS threads. Just running two greenthreads gives
    # false test pass.
    threading = eventlet.patcher.original('threading')
    addr = []

    def server():
        sock = eventlet.listen(('127.0.0.1', 0))
        addr[:] = sock.getsockname()
        eventlet.sleep(0.2)

    server_thread = threading.Thread(target=server)
    server_thread.start()
    eventlet.sleep(0.1)
    sock = eventlet.connect(tuple(addr))
    s = sock.recv(1)
    assert isinstance(s, bytes)



from eventlet import patcher
from eventlet.green import socket
from eventlet.support import six

to_patch = [('socket', socket)]

try:
    from eventlet.green import ssl
    to_patch.append(('ssl', ssl))
except ImportError:
    pass

if six.PY2:
    patcher.inject('httplib', globals(), *to_patch)
if six.PY3:
    patcher.inject('http.client', globals(), *to_patch)

if __name__ == '__main__':
    test()
